@@ -26,7 +26,6 @@
 
 #include <glade/glade.h>
 
-#include "bookmarks.h"
 #include "file-utils.h"
 #include "fr-stock.h"
 #include "main.h"
@@ -40,7 +39,7 @@ typedef struct {
 	FrWindow     *window;
 	GList        *selected_files;
 	char         *base_dir_for_selection;
-	
+
 	GtkWidget    *dialog;
 
 	GtkWidget    *e_main_vbox;
@@ -54,10 +53,6 @@ typedef struct {
 	GtkWidget    *e_password_entry;
 	GtkWidget    *e_password_hbox;
 
-	GtkWidget    *e_view_folder_checkbutton;
-
-	GtkTooltips  *tooltips;
-
 	gboolean      extract_clicked;
 } DialogData;
 
@@ -73,7 +68,6 @@ destroy_cb (GtkWidget  *widget,
 	}
 	path_list_free (data->selected_files);
 	g_free (data->base_dir_for_selection);
-	g_object_unref (data->tooltips);
 	g_free (data);
 }
 
@@ -93,6 +87,7 @@ extract_cb (GtkWidget   *w,
 	GList      *file_list;
 	char       *password;
 	char       *base_dir = NULL;
+	GError     *error = NULL;
 
 	data->extract_clicked = TRUE;
 
@@ -102,7 +97,7 @@ extract_cb (GtkWidget   *w,
 
 	/* check directory existence. */
 
-	if (! path_is_dir (extract_to_dir)) {
+	if (! uri_is_dir (extract_to_dir)) {
 		if (! ForceDirectoryCreation) {
 			GtkWidget *d;
 			int        r;
@@ -132,11 +127,11 @@ extract_cb (GtkWidget   *w,
 				do_not_extract = TRUE;
 		}
 
-		if (! do_not_extract && ! ensure_dir_exists (extract_to_dir, 0755)) {
+		if (! do_not_extract && ! ensure_dir_exists (extract_to_dir, 0755, &error)) {
 			GtkWidget  *d;
 			char       *message;
 
-			message = g_strdup_printf (_("Could not create the destination folder: %s."), gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()));
+			message = g_strdup_printf (_("Could not create the destination folder: %s."), error->message);
 
 			d = _gtk_error_dialog_new (GTK_WINDOW (window),
 						   GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -147,6 +142,7 @@ extract_cb (GtkWidget   *w,
 			gtk_widget_destroy (GTK_WIDGET (d));
 
 			g_free (message);
+			g_error_free (error);
 
 			return FALSE;
 		}
@@ -174,7 +170,7 @@ extract_cb (GtkWidget   *w,
 
 	/* check extraction directory permissions. */
 
-	if (path_is_dir (extract_to_dir)
+	if (uri_is_dir (extract_to_dir)
 	    && ! check_permissions (extract_to_dir, R_OK | W_OK | X_OK)) {
 		GtkWidget *d;
 		char      *utf8_path;
@@ -185,7 +181,7 @@ extract_cb (GtkWidget   *w,
 					   GTK_DIALOG_DESTROY_WITH_PARENT,
 					   NULL,
 					   _("Extraction not performed"),
-					   _("You don't have the right permissions to extract archives in the folder \"%s\""), 
+					   _("You don't have the right permissions to extract archives in the folder \"%s\""),
 					   utf8_path);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
@@ -199,7 +195,7 @@ extract_cb (GtkWidget   *w,
 	fr_window_set_extract_default_dir (window, extract_to_dir, TRUE);
 
 	overwrite = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_overwrite_checkbutton));
-	skip_newer = !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton)) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton));
+	skip_newer = ! gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton)) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton));
 	junk_paths = ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_recreate_dir_checkbutton));
 
 	eel_gconf_set_boolean (PREF_EXTRACT_OVERWRITE, overwrite);
@@ -220,16 +216,14 @@ extract_cb (GtkWidget   *w,
 	else
 		password = NULL;
 
-	eel_gconf_set_boolean (PREF_EXTRACT_VIEW_FOLDER, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_view_folder_checkbutton)));
-
 	/* create the file list. */
 
 	file_list = NULL;
 
-	if (selected_files){
+	if (selected_files) {
 		file_list = data->selected_files;
 		data->selected_files = NULL;       /* do not the list when destroying the dialog. */
-	} 
+	}
 	else if (pattern_files) {
 		const char *pattern;
 
@@ -256,9 +250,7 @@ extract_cb (GtkWidget   *w,
 
 	/* extract ! */
 
-	fr_window_view_folder_after_extract (window, eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE));
 	fr_window_set_password (window, password);
-
 	fr_window_archive_extract (window,
 				   file_list,
 				   extract_to_dir,
@@ -266,7 +258,7 @@ extract_cb (GtkWidget   *w,
 				   skip_newer,
 				   overwrite,
 				   junk_paths,
-				   password);
+				   TRUE);
 
 	path_list_free (file_list);
 	g_free (extract_to_dir);
@@ -386,7 +378,7 @@ create_extra_widget (DialogData *data)
 	gtk_table_attach (GTK_TABLE (table1), data->e_files_entry, 1, 2, 2, 3,
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (0), 0, 0);
-	gtk_tooltips_set_tip (data->tooltips, data->e_files_entry, _("example: *.txt; *.doc"), NULL);
+	gtk_widget_set_tooltip_text (data->e_files_entry, _("example: *.txt; *.doc"));
 	gtk_entry_set_activates_default (GTK_ENTRY (data->e_files_entry), TRUE);
 
 	data->e_all_radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("_All files"));
@@ -445,10 +437,6 @@ create_extra_widget (DialogData *data)
 	gtk_box_pack_start (GTK_BOX (data->e_password_hbox), data->e_password_entry, TRUE, TRUE, 0);
 	gtk_entry_set_activates_default (GTK_ENTRY (data->e_password_entry), TRUE);
 	gtk_entry_set_visibility (GTK_ENTRY (data->e_password_entry), FALSE);
-
-	data->e_view_folder_checkbutton = gtk_check_button_new_with_mnemonic (_("_Open destination folder after extraction"));
-	gtk_box_pack_start (GTK_BOX (vbox1), data->e_view_folder_checkbutton, FALSE, FALSE, 0);
-
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label31), data->e_password_entry);
 
 	gtk_widget_show_all (vbox1);
@@ -472,10 +460,6 @@ dlg_extract__common (FrWindow *window,
 	data->base_dir_for_selection = base_dir_for_selection;
 	data->extract_clicked = FALSE;
 
-	data->tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (data->tooltips));
-	gtk_object_sink (GTK_OBJECT (data->tooltips));
-
 	data->dialog = file_sel =
 		gtk_file_chooser_dialog_new (_("Extract"),
 					     GTK_WINDOW (data->window),
@@ -511,8 +495,6 @@ dlg_extract__common (FrWindow *window,
 	else
 		gtk_widget_set_sensitive (data->e_password_hbox, FALSE);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_view_folder_checkbutton), eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE));
-
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_overwrite_checkbutton), eel_gconf_get_boolean (PREF_EXTRACT_OVERWRITE, FALSE));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton), eel_gconf_get_boolean (PREF_EXTRACT_SKIP_NEWER, FALSE));
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_overwrite_checkbutton))) {
@@ -557,7 +539,7 @@ dlg_extract (GtkWidget *widget,
 	FrWindow *window = callback_data;
 	GList    *files;
 	char     *base_dir;
-	
+
 	files = fr_window_get_selection (window, FALSE, &base_dir);
 	dlg_extract__common (window, files, base_dir);
 }
@@ -570,7 +552,7 @@ dlg_extract_folder_from_sidebar (GtkWidget *widget,
 	FrWindow *window = callback_data;
 	GList    *files;
 	char     *base_dir;
-	
+
 	files = fr_window_get_selection (window, TRUE, &base_dir);
 	dlg_extract__common (window, files, base_dir);
 }
