@@ -25,8 +25,6 @@
 
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include "file-utils.h"
 #include "gconf-utils.h"
 #include "glib-utils.h"
@@ -66,8 +64,8 @@ open_with__destroy_cb (GtkWidget  *widget,
 {
 	g_object_unref (G_OBJECT (data->gui));
 
-	if (data->app_list)
-		gnome_vfs_mime_application_list_free (data->app_list);
+	if (data->app_list != NULL) 
+		g_list_free (data->app_list);
 
 	if (data->file_list != NULL)
 		path_list_free (data->file_list);
@@ -90,8 +88,8 @@ open_cb (GtkWidget *widget,
 	application = gtk_entry_get_text (GTK_ENTRY (data->o_app_entry));
 
 	for (scan = data->app_list; scan; scan = scan->next) {
-		GnomeVFSMimeApplication *app = scan->data;
-		if (strcmp (gnome_vfs_mime_application_get_exec (app), application) == 0) {
+		GAppInfo *app = scan->data;
+		if (strcmp (g_app_info_get_executable (app), application) == 0) {
 			fr_window_open_files_with_application (data->window, data->file_list, app);
 			gtk_widget_destroy (data->dialog);
 			return;
@@ -133,9 +131,9 @@ static void
 app_list_selection_changed_cb (GtkTreeSelection *selection,
 			       gpointer          p)
 {
-	DialogData              *data = p;
-	GtkTreeIter              iter;
-	GnomeVFSMimeApplication *app;
+	DialogData  *data = p;
+	GtkTreeIter  iter;
+	GAppInfo    *app;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->o_app_tree_view));
 	if (selection == NULL)
@@ -147,7 +145,7 @@ app_list_selection_changed_cb (GtkTreeSelection *selection,
 	gtk_tree_model_get (data->app_model, &iter,
 			    DATA_COLUMN, &app,
 			    -1);
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->o_app_entry), gnome_vfs_mime_application_get_exec (app));
+	_gtk_entry_set_locale_text (GTK_ENTRY (data->o_app_entry), g_app_info_get_executable (app));
 }
 
 
@@ -157,9 +155,9 @@ app_activated_cb (GtkTreeView       *tree_view,
 		  GtkTreeViewColumn *column,
 		  gpointer           callback_data)
 {
-	DialogData              *data = callback_data;
-	GtkTreeIter              iter;
-	GnomeVFSMimeApplication *app;
+	DialogData   *data = callback_data;
+	GtkTreeIter   iter;
+	GAppInfo     *app;
 
 	if (! gtk_tree_model_get_iter (data->app_model, &iter, path))
 		return;
@@ -168,7 +166,7 @@ app_activated_cb (GtkTreeView       *tree_view,
 			    DATA_COLUMN, &app,
 			    -1);
 
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->o_app_entry), gnome_vfs_mime_application_get_exec (app));
+	_gtk_entry_set_locale_text (GTK_ENTRY (data->o_app_entry), g_app_info_get_executable (app));
 
 	open_cb (NULL, data);
 }
@@ -275,16 +273,16 @@ void
 dlg_open_with (FrWindow *window,
 	       GList    *file_list)
 {
-	DialogData              *data;
-	GnomeVFSMimeApplication *app;
-	GList                   *scan, *app_names = NULL;
-	GSList                  *sscan, *editors;
-	GtkWidget               *cancel_button;
-	GtkTreeIter              iter;
-	GtkCellRenderer         *renderer;
-	GtkTreeViewColumn       *column;
-	GtkIconTheme            *theme;
-	int                      icon_size;
+	DialogData        *data;
+	GAppInfo          *app;
+	GList             *scan, *app_names = NULL;
+	GSList            *sscan, *editors;
+	GtkWidget         *cancel_button;
+	GtkTreeIter        iter;
+	GtkCellRenderer   *renderer;
+	GtkTreeViewColumn *column;
+	GtkIconTheme      *theme;
+	int                icon_size;
 
 	if (file_list == NULL)
 		return;
@@ -363,9 +361,9 @@ dlg_open_with (FrWindow *window,
 		const char *mime_type;
 		const char *name = scan->data;
 
-		mime_type = get_file_mime_type (name, FALSE);
-		if ((mime_type != NULL) && (strcmp (mime_type, GNOME_VFS_MIME_TYPE_UNKNOWN) != 0))
-			data->app_list = g_list_concat (data->app_list, gnome_vfs_mime_get_all_applications (mime_type));
+		mime_type = get_file_mime_type_for_path (name, FALSE);
+		if ((mime_type != NULL) && ! g_content_type_is_unknown (mime_type))
+			data->app_list = g_list_concat (data->app_list, g_app_info_get_all_for_type (mime_type));
 	}
 
 	data->app_model = GTK_TREE_MODEL (gtk_list_store_new (N_COLUMNS,
@@ -387,7 +385,8 @@ dlg_open_with (FrWindow *window,
 	for (scan = data->app_list; scan; scan = scan->next) {
 		gboolean   found;
 		char      *utf8_name;
-		GdkPixbuf *icon;
+		GIcon     *icon;
+		GdkPixbuf *icon_image;
 
 		app = scan->data;
 
@@ -395,23 +394,28 @@ dlg_open_with (FrWindow *window,
 		if (app_names != NULL) {
 			GList *p;
 			for (p = app_names; p && !found; p = p->next)
-				if (strcmp ((char*)p->data, gnome_vfs_mime_application_get_exec (app)) == 0)
+				if (strcmp ((char*)p->data, g_app_info_get_executable (app)) == 0)
 					found = TRUE;
 		}
 
 		if (found)
 			continue;
 
-		app_names = g_list_prepend (app_names, (char*) gnome_vfs_mime_application_get_exec (app));
-
-		utf8_name = g_locale_to_utf8 (app->name, -1, NULL, NULL, NULL);
-		icon = create_pixbuf (theme, gnome_vfs_mime_application_get_icon (app), icon_size);
+		app_names = g_list_prepend (app_names, (char*) g_app_info_get_executable (app));
 
+		utf8_name = g_locale_to_utf8 (g_app_info_get_name (app), -1, NULL, NULL, NULL);
+		
+		icon = g_app_info_get_icon (app);
+		if (icon != NULL) {
+			icon_image = get_icon_pixbuf (theme, G_THEMED_ICON (icon), icon_size);
+			g_object_unref (icon);
+		}
+		
 		gtk_list_store_append (GTK_LIST_STORE (data->app_model),
 				       &iter);
 		gtk_list_store_set (GTK_LIST_STORE (data->app_model),
 				    &iter,
-				    ICON_COLUMN, icon,
+				    ICON_COLUMN, icon_image,
 				    TEXT_COLUMN, utf8_name,
 				    DATA_COLUMN, app,
 				    -1);
