@@ -25,7 +25,6 @@
 
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "glib-utils.h"
 #include "file-utils.h"
 #include "gtk-utils.h"
@@ -86,7 +85,7 @@ dlg_prop (FrWindow *window)
 	GtkWidget        *label_label;
 	GtkWidget        *label;
 	char             *s;
-	GnomeVFSFileSize  size, uncompressed_size;
+	goffset           size, uncompressed_size;
 	char             *utf8_name;
 	char             *title_txt;
 	double            ratio;
@@ -108,12 +107,11 @@ dlg_prop (FrWindow *window)
 	/* Set widgets data. */
 
 	label_label = glade_xml_get_widget (data->gui, "p_path_label_label");
-	set_label (label_label, _("Path:"));
+	set_label (label_label, _("Location:"));
 
 	label = glade_xml_get_widget (data->gui, "p_path_label");
-	/* note: window->archive_filename is unescaped. */
 	s = remove_level_from_path (fr_window_get_archive_uri (window));
-	utf8_name = gnome_vfs_format_uri_for_display (s);
+	utf8_name = g_filename_display_name (s);
 	gtk_label_set_text (GTK_LABEL (label), utf8_name);
 	g_free (utf8_name);
 	g_free (s);
@@ -124,7 +122,7 @@ dlg_prop (FrWindow *window)
 	set_label (label_label, _("Name:"));
 
 	label = glade_xml_get_widget (data->gui, "p_name_label");
-	utf8_name = gnome_vfs_unescape_string_for_display (file_name_from_path (fr_window_get_archive_uri (window)));
+	utf8_name = g_uri_display_basename (fr_window_get_archive_uri (window));
 	gtk_label_set_text (GTK_LABEL (label), utf8_name);
 
 	title_txt = g_strdup_printf (_("%s Properties"), utf8_name);
@@ -150,7 +148,7 @@ dlg_prop (FrWindow *window)
 
 	label = glade_xml_get_widget (data->gui, "p_size_label");
 	size = get_file_size (fr_window_get_archive_uri (window));
-	s = gnome_vfs_format_file_size_for_display (size);
+	s = g_format_size_for_display (size);
 	gtk_label_set_text (GTK_LABEL (label), s);
 	g_free (s);
 
@@ -162,7 +160,7 @@ dlg_prop (FrWindow *window)
 	uncompressed_size = 0;
 	if (fr_window_archive_is_present (window)) {
 		int i;
-		
+
 		for (i = 0; i < window->archive->command->files->len; i++) {
 			FileData *fd = g_ptr_array_index (window->archive->command->files, i);
 			uncompressed_size += fd->size;
@@ -170,7 +168,7 @@ dlg_prop (FrWindow *window)
 	}
 
 	label = glade_xml_get_widget (data->gui, "p_uncomp_size_label");
-	s = gnome_vfs_format_file_size_for_display (uncompressed_size);
+	s = g_format_size_for_display (uncompressed_size);
 	gtk_label_set_text (GTK_LABEL (label), s);
 	g_free (s);
 
@@ -195,7 +193,7 @@ dlg_prop (FrWindow *window)
 	set_label (label_label, _("Number of files:"));
 
 	label = glade_xml_get_widget (data->gui, "p_files_label");
-	s = g_strdup_printf ("%d", window->archive->command->files->len);
+	s = g_strdup_printf ("%d", window->archive->command->n_regular_files);
 	gtk_label_set_text (GTK_LABEL (label), s);
 	g_free (s);
 
