@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2004 Free Software Foundation, Inc.
+ *  Copyright (C) 2004, 2008 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@
 #include "fr-command-7z.h"
 
 static void fr_command_7z_class_init  (FrCommand7zClass *class);
-static void fr_command_7z_init        (FrCommand         *afile);
-static void fr_command_7z_finalize    (GObject           *object);
+static void fr_command_7z_init        (FrCommand        *afile);
+static void fr_command_7z_finalize    (GObject          *object);
 
 /* Parent Class */
 
@@ -95,11 +95,28 @@ list__process_line (char     *line,
 	g_return_if_fail (line != NULL);
 
 	if (! p7z_comm->list_started) {
-		if (strncmp (line, "--------", 8) == 0)
+		if (strncmp (line, "7-Zip ", 6) == 0) {
+			const char *ver_start;
+			int         ver_len;
+		        char        version[256];
+
+			ver_start = eat_spaces (line + 6);
+		        ver_len = strchr (ver_start, ' ') - ver_start;
+        		strncpy (version, ver_start, ver_len);
+		        version[ver_len] = 0;
+
+			if (strcmp (version, "4.57") < 0)
+				p7z_comm->old_style = TRUE;
+			else
+				p7z_comm->old_style = FALSE;
+		}
+		else if (p7z_comm->old_style && (strncmp (line, "Listing archive: ", 17) == 0))
+			p7z_comm->list_started = TRUE;
+		else if (! p7z_comm->old_style && (strncmp (line, "--------", 8) == 0))
 			p7z_comm->list_started = TRUE;
 		else if (strncmp (line, "Multivolume = ", 14) == 0) {
 			fields = g_strsplit (line, " = ", 2);
-			p7z_comm->multi_volume = (strcmp (fields[1], "+") == 0);
+			comm->multi_volume = (strcmp (fields[1], "+") == 0);
 			g_strfreev (fields);
 		}
 		return;
@@ -218,7 +235,6 @@ fr_command_7z_list (FrCommand  *comm)
 		file_data_free (p7z_comm->fdata);
 		p7z_comm->fdata = NULL;
 	}
-	p7z_comm->multi_volume = FALSE;
 	p7z_comm->list_started = FALSE;
 	fr_process_start (comm->process);
 }
@@ -256,6 +272,14 @@ process_line__add (char     *line,
 {
 	FrCommand *comm = FR_COMMAND (data);
 
+	if ((comm->volume_size > 0) && (strncmp (line, "Creating archive ", 17) == 0)) {
+		char *volume_filename;
+
+		volume_filename = g_strconcat (comm->filename, ".001", NULL);
+		fr_command_set_multi_volume (comm, volume_filename);
+		g_free (volume_filename);
+	}
+
 	if (comm->n_files != 0)
 		parse_progress_line (comm, "Compressing  ", _("Adding file: "), line);
 }
@@ -494,7 +518,7 @@ fr_command_7z_get_capabilities (FrCommand  *comm,
 	}
 
 	/* multi-volumes are read-only */
-	if ((comm->files->len > 0) && FR_COMMAND_7Z (comm)->multi_volume && (capabilities & FR_COMMAND_CAN_WRITE))
+	if ((comm->files->len > 0) && comm->multi_volume && (capabilities & FR_COMMAND_CAN_WRITE))
 		capabilities ^= FR_COMMAND_CAN_WRITE;
 
 	return capabilities;
