@@ -22,12 +22,31 @@
 
 #include <config.h>
 #include <string.h>
+#include <sys/types.h>
+#include <signal.h>
 
 #include <gio/gio.h>
 #include <glade/glade.h>
 #include <libgnomeui/libgnomeui.h>
 #include "file-utils.h"
 #include "glib-utils.h"
+#include "fr-command.h"
+#include "fr-command-ace.h"
+#include "fr-command-alz.h"
+#include "fr-command-ar.h"
+#include "fr-command-arj.h"
+#include "fr-command-cfile.h"
+#include "fr-command-cpio.h"
+#include "fr-command-iso.h"
+#include "fr-command-jar.h"
+#include "fr-command-lha.h"
+#include "fr-command-rar.h"
+#include "fr-command-rpm.h"
+#include "fr-command-tar.h"
+#include "fr-command-unstuff.h"
+#include "fr-command-zip.h"
+#include "fr-command-zoo.h"
+#include "fr-command-7z.h"
 #include "fr-process.h"
 #include "fr-stock.h"
 #include "gconf-utils.h"
@@ -49,96 +68,111 @@ GList        *WindowList = NULL;
 GList        *CommandList = NULL;
 gint          ForceDirectoryCreation;
 GHashTable   *ProgramsCache = NULL;
+GPtrArray    *Registered_Commands = NULL;
 
-static gchar **remaining_args;
+static char **remaining_args;
 
-static gchar *add_to = NULL;
-static gint   add;
-static gchar *extract_to = NULL;
-static gint   extract;
-static gint   extract_here;
-static gchar *default_url = NULL;
+static char  *add_to = NULL;
+static int    add;
+static char  *extract_to = NULL;
+static int    extract;
+static int    extract_here;
+static char  *default_url = NULL;
 
-FRFileTypeDescription file_type_desc[] = {
-	{ FR_FILE_TYPE_ACE,          ".ace",     "application/x-ace", N_("Ace (.ace)"), FALSE },
-	{ FR_FILE_TYPE_AR,           ".ar",      "application/x-ar", N_("Ar (.ar)"), FALSE },
-	{ FR_FILE_TYPE_ARJ,          ".arj",     "application/x-arj", N_("Arj (.arj)"), TRUE },
-	{ FR_FILE_TYPE_BZIP,         ".bz",      "application/x-bzip", NULL, FALSE },
-	{ FR_FILE_TYPE_BZIP2,        ".bz2",     "application/x-bzip", NULL, FALSE },
-	{ FR_FILE_TYPE_COMPRESS,     ".Z",       "application/x-compress", NULL, FALSE },
-	{ FR_FILE_TYPE_CPIO,         ".cpio",    "application/x-cpio", NULL, FALSE },
-	{ FR_FILE_TYPE_DEB,          ".deb",     "application/x-deb", NULL, FALSE },
-	{ FR_FILE_TYPE_ISO,          ".iso",     "application/x-cd-image", NULL, FALSE },
-	{ FR_FILE_TYPE_EAR,          ".ear",     "application/x-ear", N_("Ear (.ear)"), TRUE },
-	{ FR_FILE_TYPE_EXE,          ".exe",     "application/x-ms-dos-executable", N_("Self-extracting zip (.exe)"), FALSE },
-	{ FR_FILE_TYPE_GZIP,         ".gz",      "application/x-gzip", NULL, FALSE },
-	{ FR_FILE_TYPE_JAR,          ".jar",     "application/x-jar", N_("Jar (.jar)"), TRUE },
-	{ FR_FILE_TYPE_LHA,          ".lzh",     "application/x-lha", N_("Lha (.lzh)"), FALSE },
-	{ FR_FILE_TYPE_LZMA,         ".lzma",    "application/x-lzma", NULL, FALSE },
-	{ FR_FILE_TYPE_LZOP,         ".lzo",     "application/x-lzop", NULL, FALSE },
-	{ FR_FILE_TYPE_RAR,          ".rar",     "application/x-rar", N_("Rar (.rar)"), TRUE },
-	{ FR_FILE_TYPE_RPM,          ".rpm",     "application/x-rpm", NULL, FALSE },
-	{ FR_FILE_TYPE_TAR,          ".tar",     "application/x-tar", N_("Tar uncompressed (.tar)"), FALSE },
-	{ FR_FILE_TYPE_TAR_BZ,       ".tar.bz",  "application/x-bzip-compressed-tar", N_("Tar compressed with bzip (.tar.bz)"), FALSE },
-	{ FR_FILE_TYPE_TAR_BZ2,      ".tar.bz2", "application/x-bzip-compressed-tar", N_("Tar compressed with bzip2 (.tar.bz2)"), FALSE },
-	{ FR_FILE_TYPE_TAR_GZ,       ".tar.gz",  "application/x-compressed-tar", N_("Tar compressed with gzip (.tar.gz)"), FALSE },
-	{ FR_FILE_TYPE_TAR_LZMA,     ".tar.lzma","application/x-lzma-compressed-tar", N_("Tar compressed with lzma (.tar.lzma)"), FALSE },
-	{ FR_FILE_TYPE_TAR_LZOP,     ".tar.lzo", "application/x-lzop-compressed-tar", N_("Tar compressed with lzop (.tar.lzo)"), FALSE },
-	{ FR_FILE_TYPE_TAR_COMPRESS, ".tar.Z",   "application/x-compressed-tar", N_("Tar compressed with compress (.tar.Z)"), FALSE },
-	{ FR_FILE_TYPE_STUFFIT,      ".sit",     "application/x-stuffit", NULL, FALSE },
-	{ FR_FILE_TYPE_WAR,          ".war",     "application/zip", N_("War (.war)"), TRUE },
-	{ FR_FILE_TYPE_ZIP,          ".zip",     "application/zip", N_("Zip (.zip)"), TRUE },
-	{ FR_FILE_TYPE_ZOO,          ".zoo",     "application/x-zoo", N_("Zoo (.zoo)"), FALSE },
-	{ FR_FILE_TYPE_7ZIP,         ".7z",      "application/x-7z-compressed", N_("7-Zip (.7z)"), TRUE },
-	{ FR_FILE_TYPE_NULL, NULL, NULL, NULL, FALSE }
+/* The capabilities are computed automatically in
+ * compute_supported_archive_types() so it's correct to initialize to 0 here. */
+FrMimeTypeDescription mime_type_desc[] = {
+	{ "application/x-7z-compressed",        ".7z",       N_("7-Zip (.7z)"), 0 },
+	{ "application/x-7z-compressed-tar",    ".tar.7z",   N_("Tar compressed with 7z (.tar.7z)"), 0 },
+	{ "application/x-ace",                  ".ace",      N_("Ace (.ace)"), 0 },
+	{ "application/x-alz",                  ".alz",      NULL, 0 },
+	{ "application/x-ar",                   ".ar",       N_("Ar (.ar)"), 0 },
+	{ "application/x-arj",                  ".arj",      N_("Arj (.arj)"), 0 },
+	{ "application/x-bzip",                 ".bz2",      NULL, 0 },
+	{ "application/x-bzip-compressed-tar",  ".tar.bz2",  N_("Tar compressed with bzip2 (.tar.bz2)"), 0 },
+	{ "application/x-bzip1",                ".bz",       NULL, 0 },
+	{ "application/x-bzip1-compressed-tar", ".tar.bz",   N_("Tar compressed with bzip (.tar.bz)"), 0 },
+	{ "application/x-cabinet",              ".cab",      N_("Cabinet (.cab)"), 0 },
+	{ "application/x-cbr",                  ".cbr",      N_("Rar Archived Comic Book (.cbr)"), 0 },
+	{ "application/x-cbz",                  ".cbz",      N_("Zip Archived Comic Book (.cbz)"), 0 },
+	{ "application/x-cd-image",             ".iso",      NULL, 0 },
+	{ "application/x-compress",             ".Z",        NULL, 0 },
+	{ "application/x-compressed-tar",       ".tar.gz",   N_("Tar compressed with gzip (.tar.gz)"), 0 },
+	{ "application/x-cpio",                 ".cpio",     NULL, 0 },
+	{ "application/x-deb",                  ".deb",      NULL, 0 },
+	{ "application/x-ear",                  ".ear",      N_("Ear (.ear)"), 0 },
+	{ "application/x-executable",           ".exe",      N_("Self-extracting zip (.exe)"), 0 },
+	{ "application/x-gzip",                 ".gz",       NULL, 0 },
+	{ "application/x-java-archive",         ".jar",      N_("Jar (.jar)"), 0 },
+	{ "application/x-lha",                  ".lzh",      N_("Lha (.lzh)"), 0 },
+	{ "application/x-lzma",                 ".lzma",     NULL, 0 },
+	{ "application/x-lzma-compressed-tar",  ".tar.lzma", N_("Tar compressed with lzma (.tar.lzma)"), 0 },
+	{ "application/x-lzop",                 ".lzo",      NULL, 0 },
+	{ "application/x-lzop-compressed-tar",  ".tar.lzo",  N_("Tar compressed with lzop (.tar.lzo)"), 0 },
+	{ "application/x-rar",                  ".rar",      N_("Rar (.rar)"), 0 },
+	{ "application/x-rpm",                  ".rpm",      NULL, 0 },
+	{ "application/x-rzip",                 ".rz",       NULL, 0 },
+	{ "application/x-tar",                  ".tar",      N_("Tar uncompressed (.tar)"), 0 },
+	{ "application/x-tarz",                 ".tar.Z",    N_("Tar compressed with compress (.tar.Z)"), 0 },
+	{ "application/x-stuffit",              ".sit",      NULL, 0 },
+	{ "application/x-war",                  ".war",      N_("War (.war)"), 0 },
+	{ "application/x-zoo",                  ".zoo",      N_("Zoo (.zoo)"), 0 },
+	{ "application/zip",                    ".zip",      N_("Zip (.zip)"), 0 },
+	{ NULL, NULL, NULL, 0 }
 };
 
-
-FRFileType single_file_save_type[32];
-FRFileType save_type[32];
-FRFileType open_type[32];
-
-
-FRCommandDescription command_desc[] = {
-	{ "tar",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR },
-	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ZIP },
-	{ "unzip",      TRUE,  FALSE, TRUE,  FR_FILE_TYPE_ZIP },
-	{ "rar",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_RAR },
-	{ "unrar",      TRUE,  FALSE, TRUE,  FR_FILE_TYPE_RAR },
-	{ "gzip",       TRUE,  TRUE,  FALSE, FR_FILE_TYPE_GZIP },
-	{ "bzip2",      TRUE,  TRUE,  FALSE, FR_FILE_TYPE_BZIP2 },
-	{ "unace",      TRUE,  FALSE, TRUE,  FR_FILE_TYPE_ACE },
-	{ "ar",         TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_AR },
-	{ "ar",         TRUE,  FALSE, TRUE,  FR_FILE_TYPE_DEB },
-	{ "arj",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ARJ },
-	{ "bzip2",      TRUE,  FALSE, FALSE, FR_FILE_TYPE_BZIP },
-	{ "compress",   TRUE,  TRUE,  FALSE, FR_FILE_TYPE_COMPRESS },
-	{ "cpio",       TRUE,  FALSE, FALSE, FR_FILE_TYPE_CPIO },
-	{ "isoinfo",    TRUE,  FALSE, TRUE,  FR_FILE_TYPE_ISO },
-	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_EAR },
-	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_JAR },
-	{ "zip",        TRUE,  FALSE,  TRUE,  FR_FILE_TYPE_EXE },
-	{ "lha",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_LHA },
-	{ "lzma",       TRUE,  TRUE,  FALSE, FR_FILE_TYPE_LZMA },
-	{ "lzop",       TRUE,  TRUE,  FALSE, FR_FILE_TYPE_LZOP },
-	{ "rpm2cpio",   TRUE,  FALSE, TRUE,  FR_FILE_TYPE_RPM },
-	{ "uncompress", TRUE,  FALSE, FALSE, FR_FILE_TYPE_COMPRESS },
-	{ "unstuff",    TRUE,  FALSE, FALSE, FR_FILE_TYPE_STUFFIT },
-	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_WAR },
-	{ "zoo",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ZOO },
-	{ "7za",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_7ZIP },
-	{ "7zr",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_7ZIP }
-};
-
-FRCommandDescription tar_command_desc[] = {
-	{ "gzip",      TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_GZ },
-	{ "bzip2",     TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_BZ2 },
-	{ "bzip",      FALSE, TRUE,  TRUE,  FR_FILE_TYPE_TAR_BZ },
-	{ "lzma",      TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_LZMA },
-	{ "lzop",      TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_LZOP },
-	{ "compress",  TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_COMPRESS }
+FrExtensionType file_ext_type[] = {
+	{ ".7z", "application/x-7z-compressed" },
+	{ ".ace", "application/x-ace" },
+	{ ".alz", "application/x-alz" },
+	{ ".ar", "application/x-ar" },
+	{ ".arj", "application/x-arj" },
+	{ ".bin", "application/x-stuffit" },
+	{ ".bz", "application/x-bzip" },
+	{ ".bz2", "application/x-bzip" },
+	{ ".cab", "application/x-cabinet" },
+	{ ".cbr", "application/x-cbr" },
+	{ ".cbz", "application/x-cbz" },
+	{ ".cpio", "application/x-cpio" },
+	{ ".deb", "application/x-deb" },
+	{ ".ear", "application/x-ear" },
+	{ ".exe", "application/x-executable" },
+	{ ".gz", "application/x-gzip" },
+	{ ".iso", "application/x-cd-image" },
+	{ ".jar", "application/x-java-archive" },
+	{ ".lha", "application/x-lha" },
+	{ ".lzh", "application/x-lha" },
+	{ ".lzma", "application/x-lzma" },
+	{ ".lzo", "application/x-lzop" },
+	{ ".rar", "application/x-rar" },
+	{ ".rpm", "application/x-rpm" },
+	{ ".rz", "application/x-rzip" },
+	{ ".sit", "application/x-stuffit" },
+	{ ".tar", "application/x-tar" },
+	{ ".tar.bz", "application/x-bzip-compressed-tar" },
+	{ ".tar.bz2", "application/x-bzip-compressed-tar" },
+	{ ".tar.gz", "application/x-compressed-tar" },
+	{ ".tar.lzma", "application/x-lzma-compressed-tar" },
+	{ ".tar.lzo", "application/x-lzop-compressed-tar" },
+	{ ".tar.7z", "application/x-7z-compressed-tar" },
+	{ ".tar.Z", "application/x-tarz" },
+	{ ".taz", "application/x-tarz" },
+	{ ".tbz", "application/x-bzip-compressed-tar" },
+	{ ".tbz2", "application/x-bzip-compressed-tar" },
+	{ ".tgz", "application/x-compressed-tar" },
+	{ ".tzma", "application/x-lzma-compressed-tar" },
+	{ ".tzo", "application/x-lzop-compressed-tar" },
+	{ ".war", "application/x-war" },
+	{ ".z", "application/x-gzip" },
+	{ ".Z", "application/x-compress" },
+	{ ".zip", "application/zip" },
+	{ ".zoo", "application/x-zoo" }
 };
 
+int single_file_save_type[64];
+int save_type[64];
+int open_type[64];
+int create_type[64];
 
 static const GOptionEntry options[] = {
 	{ "add-to", 'a', 0, G_OPTION_ARG_STRING, &add_to,
@@ -209,9 +243,9 @@ main (int argc, char **argv)
 	context = g_option_context_new (N_("- Create and modify an archive"));
 	g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-	
+
 	program = gnome_program_init ("file-roller", VERSION,
-				      LIBGNOMEUI_MODULE, 
+				      LIBGNOMEUI_MODULE,
 				      argc, argv,
 				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Archive Manager"),
@@ -261,7 +295,7 @@ command_done (CommandData *cdata)
 {
 	if (cdata == NULL)
 		return;
-		
+
 	if ((cdata->temp_dir != NULL) && path_is_dir (cdata->temp_dir)) {
 		char *argv[4];
 
@@ -320,8 +354,8 @@ migrate_dir_from_to (const char *from_dir,
 		char *e1;
 		char *e2;
 
-		e1 = shell_escape (from_path);
-		e2 = shell_escape (to_path);
+		e1 = g_shell_quote (from_path);
+		e2 = g_shell_quote (to_path);
 		line = g_strdup_printf ("mv -f %s %s", e1, e2);
 		g_free (e1);
 		g_free (e2);
@@ -350,8 +384,8 @@ migrate_file_from_to (const char *from_file,
 		char *e1;
 		char *e2;
 
-		e1 = shell_escape (from_path);
-		e2 = shell_escape (to_path);
+		e1 = g_shell_quote (from_path);
+		e2 = g_shell_quote (to_path);
 		line = g_strdup_printf ("mv -f %s %s", e1, e2);
 		g_free (e1);
 		g_free (e2);
@@ -376,40 +410,266 @@ migrate_to_new_directories (void)
 }
 
 
+/* -- FrRegisteredCommand -- */
+
+
+FrRegisteredCommand *
+fr_registered_command_new (GType command_type)
+{
+	FrRegisteredCommand  *reg_com;
+	FrCommand            *command;
+	const char          **mime_types;
+	int                   i;
+
+	reg_com = g_new0 (FrRegisteredCommand, 1);
+	reg_com->ref = 1;
+	reg_com->type = command_type;
+	reg_com->caps = g_ptr_array_new ();
+
+	command = (FrCommand*) g_object_new (reg_com->type, NULL);
+	mime_types = fr_command_get_mime_types (command);
+	for (i = 0; mime_types[i] != NULL; i++) {
+		const char    *mime_type;
+		FrMimeTypeCap *cap;
+
+		mime_type = get_static_string (mime_types[i]);
+
+		cap = g_new0 (FrMimeTypeCap, 1);
+		cap->mime_type = mime_type;
+		cap->capabilities = fr_command_get_capabilities (command, mime_type);
+		g_ptr_array_add (reg_com->caps, cap);
+	}
+	g_object_unref (command);
+
+	return reg_com;
+}
+
+
+void
+fr_registered_command_ref (FrRegisteredCommand *reg_com)
+{
+	reg_com->ref++;
+}
+
+
+void
+fr_registered_command_unref (FrRegisteredCommand *reg_com)
+{
+	if (--(reg_com->ref) != 0)
+		return;
+
+	g_ptr_array_foreach (reg_com->caps, (GFunc) g_free, NULL);
+	g_ptr_array_free (reg_com->caps, TRUE);
+	g_free (reg_com);
+}
+
+
+FrCommandCaps
+fr_registered_command_get_capabilities (FrRegisteredCommand *reg_com,
+				        const char          *mime_type)
+{
+	int i;
+
+	for (i = 0; i < reg_com->caps->len; i++) {
+		FrMimeTypeCap *cap;
+
+		cap = g_ptr_array_index (reg_com->caps, i);
+		if (strcmp (mime_type, cap->mime_type) == 0)
+			return cap->capabilities;
+	}
+
+	return FR_COMMAND_CAN_DO_NOTHING;
+}
+
+
+void
+register_command (GType command_type)
+{
+	if (Registered_Commands == NULL)
+		Registered_Commands = g_ptr_array_sized_new (5);
+	g_ptr_array_add (Registered_Commands, fr_registered_command_new (command_type));
+}
+
+
+gboolean
+unregister_command (GType command_type)
+{
+	int i;
+
+	for (i = 0; i < Registered_Commands->len; i++) {
+		FrRegisteredCommand *command;
+
+		command = g_ptr_array_index (Registered_Commands, i);
+		if (command->type == command_type) {
+			g_ptr_array_remove_index (Registered_Commands, i);
+			fr_registered_command_unref (command);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
+static void
+register_commands (void)
+{
+	/* The order here is important. Commands registered earlier have higher
+	 * priority; for example zip archives will be opened using 7Z instead
+	 * of ZIP.  However commands that can read and write a file format
+	 * have higher priority over commands that can only read the same
+	 * format, regardless of the registration order; for example rar
+	 * archives will be opened with 7Z only if the rar utility is not
+	 * installed because 7z cannot modify rar archives. */
+
+	register_command (FR_TYPE_COMMAND_TAR);
+	register_command (FR_TYPE_COMMAND_CFILE);
+	register_command (FR_TYPE_COMMAND_7Z);
+
+	register_command (FR_TYPE_COMMAND_ACE);
+	register_command (FR_TYPE_COMMAND_ALZ);
+	register_command (FR_TYPE_COMMAND_AR);
+	register_command (FR_TYPE_COMMAND_ARJ);
+	register_command (FR_TYPE_COMMAND_CPIO);
+	register_command (FR_TYPE_COMMAND_ISO);
+	register_command (FR_TYPE_COMMAND_JAR);
+	register_command (FR_TYPE_COMMAND_LHA);
+	register_command (FR_TYPE_COMMAND_RAR);
+	register_command (FR_TYPE_COMMAND_RPM);
+	register_command (FR_TYPE_COMMAND_UNSTUFF);
+	register_command (FR_TYPE_COMMAND_ZIP);
+	register_command (FR_TYPE_COMMAND_ZOO);
+}
+
+
+GType
+get_command_type_from_mime_type (const char    *mime_type,
+				 FrCommandCaps  requested_capabilities)
+{
+	int i;
+
+	if (mime_type == NULL)
+		return 0;
+
+	for (i = 0; i < Registered_Commands->len; i++) {
+		FrRegisteredCommand *command;
+		FrCommandCaps        capabilities;
+
+		command = g_ptr_array_index (Registered_Commands, i);
+		capabilities = fr_registered_command_get_capabilities (command, mime_type);
+
+		/* the command must support all the requested capabilities */
+		if (((capabilities ^ requested_capabilities) & requested_capabilities) == 0)
+			return command->type;
+	}
+
+	return 0;
+}
+
+
+const char *
+get_mime_type_from_extension (const char *ext)
+{
+	int i;
+
+	if (ext == NULL)
+		return NULL;
+
+	for (i = G_N_ELEMENTS (file_ext_type) - 1; i >= 0; i--)
+		if (strcasecmp (ext, file_ext_type[i].ext) == 0)
+			return get_static_string (file_ext_type[i].mime_type);
+
+	return NULL;
+}
+
+
+const char *
+get_archive_filename_extension (const char *filename)
+{
+	const char *ext;
+	int         i;
+
+	if (filename == NULL)
+		return NULL;
+
+	ext = get_file_extension (filename);
+	if (ext == NULL)
+		return NULL;
+
+	for (i = G_N_ELEMENTS (file_ext_type) - 1; i >= 0; i--)
+		if (strcasecmp (ext, file_ext_type[i].ext) == 0)
+			return ext;
+	return NULL;
+}
+
+
+int
+get_mime_type_index (const char *mime_type)
+{
+	int i;
+
+	for (i = 0; mime_type_desc[i].mime_type != NULL; i++)
+		if (strcmp (mime_type_desc[i].mime_type, mime_type) == 0)
+			return i;
+	return -1;
+}
+
+
+static void
+add_if_non_present (int *a,
+	            int *n,
+	            int  o)
+{
+	int i;
+
+	for (i = 0; i < *n; i++) {
+		if (a[i] == o)
+			return;
+	}
+	a[*n] = o;
+	*n = *n + 1;
+}
+
+
 static void
 compute_supported_archive_types (void)
 {
-	int i, j;
-	int sf_i = 0, s_i = 0, o_i = 0;
+	int sf_i = 0, s_i = 0, o_i = 0, c_i = 0;
+	int i;
 
-	for (i = 0; i < G_N_ELEMENTS (command_desc); i++) {
-		FRCommandDescription com = command_desc[i];
+	for (i = 0; i < Registered_Commands->len; i++) {
+		FrRegisteredCommand *reg_com;
+		int                  j;
 
-		if (!is_program_in_path (com.command))
-			continue;
+		reg_com = g_ptr_array_index (Registered_Commands, i);
+		for (j = 0; j < reg_com->caps->len; j++) {
+			FrMimeTypeCap *cap;
+			int            idx;
 
-		if (strcmp (com.command, "tar") == 0)
-			for (j = 0; j < G_N_ELEMENTS (tar_command_desc); j++) {
-				FRCommandDescription com2 = tar_command_desc[j];
-
-				if (!is_program_in_path (com2.command))
-					continue;
-				open_type[o_i++] = com2.file_type;
-				save_type[s_i++] = com2.file_type;
-				single_file_save_type[sf_i++] = com2.file_type;
+			cap = g_ptr_array_index (reg_com->caps, j);
+			idx = get_mime_type_index (cap->mime_type);
+			if (idx < 0) {
+				g_warning ("mime type not recognized: %s", cap->mime_type);
+				continue;
 			}
-
-		if (com.can_open)
-			open_type[o_i++] = com.file_type;
-		if (com.can_save && com.support_many_files)
-			save_type[s_i++] = com.file_type;
-		if (com.can_save)
-			single_file_save_type[sf_i++] = com.file_type;
+			mime_type_desc[idx].capabilities |= cap->capabilities;
+			if (cap->capabilities & FR_COMMAND_CAN_READ)
+				add_if_non_present (open_type, &o_i, idx);
+			if (cap->capabilities & FR_COMMAND_CAN_WRITE) {
+				if (cap->capabilities & FR_COMMAND_CAN_ARCHIVE_MANY_FILES) {
+					add_if_non_present (save_type, &s_i, idx);
+					if (cap->capabilities & FR_COMMAND_CAN_WRITE)
+						add_if_non_present (create_type, &c_i, idx);
+				}
+				add_if_non_present (single_file_save_type, &sf_i, idx);
+			}
+		}
 	}
 
-	open_type[o_i++] = FR_FILE_TYPE_NULL;
-	save_type[s_i++] = FR_FILE_TYPE_NULL;
-	single_file_save_type[sf_i++] = FR_FILE_TYPE_NULL;
+	open_type[o_i] = -1;
+	save_type[s_i] = -1;
+	single_file_save_type[sf_i] = -1;
+	create_type[c_i] = -1;
 }
 
 
@@ -426,7 +686,7 @@ get_uri_from_command_line (const char *path)
 		full_path = g_strdup (path);
 	else {
 		char *current_dir;
-		
+
 		current_dir = g_get_current_dir ();
 		full_path = g_build_filename (current_dir,
 					      path,
@@ -451,10 +711,10 @@ prepare_app (void)
 	/* create the config dir if necessary. */
 
 	uri = get_home_relative_uri (RC_DIR);
-	
+
 	if (uri_is_file (uri)) { /* before the gconf port this was a file, now it's folder. */
 		GFile *file;
-		
+
 		file = g_file_new_for_uri (uri);
 		g_file_delete (file, NULL, NULL);
 		g_object_unref (file);
@@ -466,6 +726,7 @@ prepare_app (void)
 	if (eel_gconf_get_boolean (PREF_MIGRATE_DIRECTORIES, TRUE))
 		migrate_to_new_directories ();
 
+	register_commands ();
 	compute_supported_archive_types ();
 
 	if (session_is_restored ()) {
@@ -531,18 +792,13 @@ prepare_app (void)
 				fr_window_set_batch__extract (FR_WINDOW (window),
 							      archive_uri,
 							      extract_to_path);
-			g_free (archive_uri); 
+			g_free (archive_uri);
 		}
 		fr_window_append_batch_action (FR_WINDOW (window),
 					       FR_BATCH_ACTION_QUIT,
 					       NULL,
 					       NULL);
 
-		if (eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE))
-			/* open the destination folder only if we are 
-			 * extracting a single archive. */					       
-			fr_window_view_folder_after_extract (FR_WINDOW (window), (i - 1 == 1));
-					       
 		fr_window_start_batch (FR_WINDOW (window));
 	}
 	else { /* Open each archive in a window */
@@ -553,10 +809,10 @@ prepare_app (void)
 			GtkWidget *window;
 			GFile     *file;
 			char      *uri;
-			
+
 			window = fr_window_new ();
 			gtk_widget_show (window);
-			
+
 			file = g_file_new_for_commandline_arg (filename);
 			uri = g_file_get_uri (file);
 			fr_window_archive_open (FR_WINDOW (window), uri, GTK_WINDOW (window));
@@ -595,7 +851,7 @@ save_session (GnomeClient *client)
 	int       i;
 	char     *path;
 	GFile    *file;
-	
+
 	key_file = g_key_file_new ();
 	i = 0;
 	for (scan = WindowList; scan; scan = scan->next) {
@@ -608,7 +864,7 @@ save_session (GnomeClient *client)
 		}
 		else {
 			char *uri;
-			
+
 			uri = g_file_get_uri (window->archive->file);
 			g_key_file_set_string (key_file, "Sessione", key, uri);
 			g_free (uri);
@@ -617,12 +873,12 @@ save_session (GnomeClient *client)
 
 		i++;
 	}
-	g_key_file_set_integer (key_file, "Sessione", "archives", i); 
-	
+	g_key_file_set_integer (key_file, "Sessione", "archives", i);
+
 	path = get_real_path_for_prefix (gnome_client_get_config_prefix (client));
 	file = g_file_new_for_path (path);
 	g_key_file_save (key_file, file);
-	
+
 	g_object_unref (file);
 	g_free (path);
 	g_key_file_free (key_file);
