@@ -177,13 +177,13 @@ static GtkActionEntry action_entries[] = {
 	  N_("_Test Integrity"), NULL,
 	  N_("Test whether the archive contains errors"),
 	  G_CALLBACK (activate_action_test_archive) },
-	{ "ViewSelection", FR_STOCK_VIEW,
+	{ "ViewSelection", GTK_STOCK_OPEN,
 	  NULL, NULL,
-	  N_("View the selected file"),
+	  N_("Open the selected file"),
 	  G_CALLBACK (activate_action_view_or_open) },
-	{ "ViewSelection_Toolbar", FR_STOCK_VIEW,
+	{ "ViewSelection_Toolbar", GTK_STOCK_OPEN,
 	  NULL, NULL,
-	  N_("View the selected file"),
+	  N_("Open the selected file"),
 	  G_CALLBACK (activate_action_view_or_open) },
 	{ "OpenFolder", GTK_STOCK_OPEN,
 	  NULL, NULL,
@@ -200,7 +200,7 @@ static GtkActionEntry action_entries[] = {
 	  G_CALLBACK (activate_action_go_back) },
 	{ "GoForward", GTK_STOCK_GO_FORWARD,
 	  NULL, NULL,
-	  N_("Open Go to the next visited location selected folder"),
+	  N_("Go to the next visited location"),
 	  G_CALLBACK (activate_action_go_forward) },
 	{ "GoUp", GTK_STOCK_GO_UP,
 	  NULL, NULL,
